/*
 * codec_func.h
 *
 * implements the "analog" operating mode of DVfirmware. Direct i/o with
 * the speech codec AD73311 (w/o using AMBE-2020).
 *
 *
 *  Created on: 19.09.2009
 *      Author: DO1FJN, Jan Alte
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef CODEC_FUNC_H_
#define CODEC_FUNC_H_

typedef enum {
  CODECnoinit, CODECoff, CODECtransmit, CODECreceive
} tcodecstate;

typedef struct __attribute__((__packed__)) {	// Setup-Paket
  unsigned char funcsel;	// Funktionsauswahl RX (LowNibble), TX
  unsigned char volume;		// Lautstärke
  unsigned char micgain;	// Mikrofonverstärkung +/- 16dB
  unsigned char tonevol;	// Mithörlautstärke Töne/DTMF
  unsigned char tonehub;	// Tone-Sende-Hub (Lautstärke, Log)
  unsigned char hublimit;	// Sende-Hub Limit (Linearer Wert 255 ^= Unlimit)
  unsigned char sqlevel;	// Squelch-Offen-Wert
  unsigned char	reserved;
} tcodecsetup;

#define VOLUME_SCALE		160	// 16dB Regelbereich 0.5dB schritte
#define MICGAIN_SCALE		160	// 16dB Regelbereich

void codec_init(void);
void codec_exit(void);
void codec_transmit(void);
void codec_receive(void);
void codec_mute(void);		// Mute Receiving (TX no effect) until codec_receive()
void codec_unmute(void);

void codec_gentone(int freq1, int freq2, int ms);
void codec_calltone(void);
void codec_toneoff(void);

void codec_setup(tcodecsetup *Setup);

void codec_set_volume(unsigned char vol);
void codec_set_micgain(unsigned char gain);

tcodecstate codec_getstate(void);

#endif // CODEC_FUNC_H_
