/*
 * bf_func.h
 *
 * defines timing and structures of the internal serial bus communication (BF-Bus).
 * Include API functions to communicate with DVUP (Siemens C5 main processor).
 *
 *
 *  Created on: 07.03.2009
 *      Author: Jan Alte, DO1FJN
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef BF_FUNC_H_
#define BF_FUNC_H_

#define BF_BAUDRATE	19200L

#define BF_MYADDRESS	0x44
#define BF_MASTER	0x00		// C5 Main-µP als Destination

#define BFRXBUFSIZE	4		// Anzahl Empfangs-Buffer
#define BFTXBUFSIZE	4		// Anzahl Sende-Buffer

#define BFByteTime	11
#define BFBusPktFinTO	26		// 1.968ms (37.79)
#define BFBusTestTime	42		// orig. 0.984ms, 1.5Byte...
#define BFBusPktEarlier	26		// 24, Zeit vor RX-To
#define BFBusBusyTime	304		// 306 = 15.96ms Zeitdauer in Bits
#define BFBusBreakTime  89		// 4.63ms


#define BF_TelegrSize	29		// Max. Größe eines Telegrammes...
// (die max. Größe entspricht der in Siemens C5 ^= Maximum)
typedef struct __attribute__((__packed__)) {	// Aufbau eines Telegrammes
  unsigned char fromadr;
  unsigned char destadr;
  unsigned char pktlen;
  unsigned char cmd;
  unsigned char data[BF_TelegrSize-4];
} tbfpacket;


typedef enum {
  BFerr_txoverflow, BFerr_rxoverflow,
  BFerr_transmit, BFerr_txData, BFerr_ivalidTOidle,
  BFerr_noTxD, BFerr_noACK
} tBFerror;


typedef void (*tBFerrorhandler)(tBFerror Error);



void bf_init(void);
void bf_set_errorfct(tBFerrorhandler ErrorFunction);


unsigned char	bf_pktreceived(void);
void		bf_copyreceivedpkt(tbfpacket *dest);
tbfpacket	*bf_getreceivedpkt(void);
void		bf_freereceivedpkt(void);

void		bf_sendpkt(const tbfpacket *Packet);
void		bf_send2C5(char Cmd, char Len, const char *Data);

unsigned char	bf_getcrc(const tbfpacket *Packet);


#endif
